from django.shortcuts import render,redirect
from django.contrib.auth.decorators import user_passes_test
from django.utils.decorators import method_decorator
from rest_framework.response import Response
from rest_framework import status
from rest_framework.views import APIView
# Create your views here.
from .serializers import *
from .models import *
#from datetime import timedelta,date
from brpatientApp.models import Patient
from brvisitApp.models import PatientVisit
#import json


############## END PatientVisit%Module #################
############## Prescription%Module #################
###### Drugs الأدوية########
class DrugsApi(APIView):
    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.view_drugs'), login_url='/main/login/'))
    def get(self,request):

        all_drugs =Drug.objects.filter(is_active=True)
        all_drugs_serialized=DrugSerializer(all_drugs,many=True)
        time_todos =TimeTodo.objects.filter(is_active=True).values('id','name')
        return render(request,"Branch/brpresApp/drugs.html",{"all_drugs":all_drugs_serialized.data,
                                                             "time_todos":time_todos,
                                                             'activePage':'Drugs'})
    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.add_drug'), login_url='/main/login/'))    
    def post(self,request):
        data = request.data
        data._mutable = True
        serializedObj= DrugSerializer(data=data)
        #list_error=""
        if serializedObj.is_valid():
            serializedObj.save()
            return Response(serializedObj.data,status=status.HTTP_201_CREATED)
        else :return Response(serializedObj.errors,status=status.HTTP_400_BAD_REQUEST)
class DrugApi(APIView):
    def get_object(self,pk):
        try:
            return Drug.objects.get(pk=pk)
        except Drug.DoesNotExist:
            return None    
    
    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.view_drugs'), login_url='/main/login/'))        
    def get(self,request,pk): 
        drugObj=self.get_object(pk)
        if drugObj :
            #return render(request,"mainApp/drug.html",{"drug":drugObj,'activePage':'Drugs'})
            serializeobj= DrugSerializer(drugObj)
            return Response(serializeobj.data,status=status.HTTP_200_OK)
        else : return Response(status=status.HTTP_404_NOT_FOUND)

    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.change_drug'), login_url='/main/login/'))
    def put(self,request,pk):
        obj=self.get_object(pk)
        serializedObj= DrugSerializer(obj,data=request.data,partial=True)
        #list_error=""
        if serializedObj.is_valid():
            serializedObj.save()
            return Response(status=status.HTTP_200_OK)
        else : return Response(serializedObj.errors,status=status.HTTP_400_BAD_REQUEST)

    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.delete_drug'), login_url='/main/login/'))
    def delete(self,request,pk):
        obj=self.get_object(pk)
        if obj :
            try:
                obj.delete()
                return Response(status=status.HTTP_204_NO_CONTENT)
            except Exception as err_msg: 
                    return Response(data ="{}".format(str(err_msg)),status=status.HTTP_405_METHOD_NOT_ALLOWED)
        else : return Response(status=status.HTTP_404_NOT_FOUND)
###### Radiologys   الأشعة########
class RadiologysApi(APIView):
    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.view_radiologys'), login_url='/main/login/'))
    def get(self,request):
        all_radiologys =Radiology.objects.filter(is_active=True)
        all_radiologys_serialized=RadiologySerializer(all_radiologys,many=True)
        time_todos =TimeTodo.objects.filter(is_active=True).values('id','name')
        return render(request,"Branch/brpresApp/radiologys.html",{"all_radiologys":all_radiologys_serialized.data,
                                                                  "time_todos":time_todos,
                                                                  'activePage':'Radiologys'})
    
    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.add_radiology'), login_url='/main/login/'))    
    def post(self,request):
        data = request.data
        data._mutable = True
        data.update({"is_active":True})
        serializedObj= RadiologySerializer(data=data)
        if serializedObj.is_valid():
            serializedObj.save()
            return Response(serializedObj.data,status=status.HTTP_201_CREATED)
        else :return Response(serializedObj.errors,status=status.HTTP_400_BAD_REQUEST)
class RadiologyApi(APIView):
    def get_object(self,pk):
        try:
            return Radiology.objects.get(pk=pk)
        except Radiology.DoesNotExist:
            return None    
    
    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.view_radiologys'), login_url='/main/login/'))        
    def get(self,request,pk): 
        radiologyObj=self.get_object(pk)
        if radiologyObj :
            #return render(request,"mainApp/radiology.html",{"radiology":radiologyObj,'activePage':'Radiologys'})
            serializeobj= RadiologySerializer(radiologyObj)
            return Response(serializeobj.data,status=status.HTTP_200_OK)
        else : return Response(status=status.HTTP_404_NOT_FOUND)

    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.change_radiology'), login_url='/main/login/'))
    def put(self,request,pk):
        obj=self.get_object(pk)
        serializedObj= RadiologySerializer(obj,data=request.data,partial=True)
        #list_error=""
        if serializedObj.is_valid():
            serializedObj.save()
            return Response(status=status.HTTP_200_OK)
        else : return Response(serializedObj.errors,status=status.HTTP_400_BAD_REQUEST)

    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.delete_radiology'), login_url='/main/login/'))
    def delete(self,request,pk):
        obj=self.get_object(pk)
        if obj :
            try:
                obj.delete()
                return Response(status=status.HTTP_204_NO_CONTENT)
            except Exception as err_msg: 
                    return Response(data ="{}".format(str(err_msg)),status=status.HTTP_405_METHOD_NOT_ALLOWED)
        else : return Response(status=status.HTTP_404_NOT_FOUND)
###### MedicalTests   التحاليل الطبية########
class MedicalTestsApi(APIView):
    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.view_medicaltests'), login_url='/main/login/'))
    def get(self,request):
        all_medical_tests =MedicalTest.objects.filter(is_active=True)
        all_medical_tests_serialized=MedicalTestSerializer(all_medical_tests,many=True)
        time_todos =TimeTodo.objects.filter(is_active=True).values('id','name')
        return render(request,"Branch/brpresApp/medical_tests.html",{"all_medical_tests":all_medical_tests_serialized.data,
                                                                     "time_todos":time_todos,
                                                                     'activePage':'MedicalTests'})
    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.add_medicaltest'), login_url='/main/login/'))    
    def post(self,request):
        data = request.data
        data._mutable = True
        data.update({"is_active":True})
        serializedObj= MedicalTestSerializer(data=data)
        if serializedObj.is_valid():
            serializedObj.save()
            return Response(serializedObj.data,status=status.HTTP_201_CREATED)
        else :return Response(serializedObj.errors,status=status.HTTP_400_BAD_REQUEST)
class MedicalTestApi(APIView):
    def get_object(self,pk):
        try:
            return MedicalTest.objects.get(pk=pk)
        except MedicalTest.DoesNotExist:
            return None    
    
    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.view_medicaltests'), login_url='/main/login/'))        
    def get(self,request,pk): 
        medical_testObj=self.get_object(pk)
        if medical_testObj :
            #return render(request,"mainApp/medical_test.html",{"medical_test":medical_testObj,'activePage':'MedicalTests'})
            serializeobj= MedicalTestSerializer(medical_testObj)
            return Response(serializeobj.data,status=status.HTTP_200_OK)
        else : return Response(status=status.HTTP_404_NOT_FOUND)

    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.change_medicaltest'), login_url='/main/login/'))
    def put(self,request,pk):
        obj=self.get_object(pk)
        serializedObj= MedicalTestSerializer(obj,data=request.data,partial=True)
        #list_error=""
        if serializedObj.is_valid():
            serializedObj.save()
            return Response(status=status.HTTP_200_OK)
        else : return Response(serializedObj.errors,status=status.HTTP_400_BAD_REQUEST)

    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.delete_medicaltest'), login_url='/main/login/'))
    def delete(self,request,pk):
        obj=self.get_object(pk)
        if obj :
            try:
                obj.delete()
                return Response(status=status.HTTP_204_NO_CONTENT)
            except Exception as err_msg: 
                    return Response(data ="{}".format(str(err_msg)),status=status.HTTP_405_METHOD_NOT_ALLOWED)
        else : return Response(status=status.HTTP_404_NOT_FOUND)
##### TimeTodos  توقيتات الإستخدام########
class TimeTodosApi(APIView):
    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.view_timetodos'), login_url='/main/login/'))
    def get(self,request):
        all_time_todos =TimeTodo.objects.filter(is_active=True)
        all_time_todos_serialized=TimeTodoSerializer(all_time_todos,many=True)
        return render(request,"Branch/brpresApp/time_todos.html",{"all_time_todos":all_time_todos_serialized.data,'activePage':'TimeTodos'})
    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.add_timetodo'), login_url='/main/login/'))    
    def post(self,request):
        data = request.data
        data._mutable = True
        data.update({"is_active":True})
        serializedObj= TimeTodoSerializer(data=data)
        if serializedObj.is_valid():
            serializedObj.save()
            return Response(serializedObj.data,status=status.HTTP_201_CREATED)
        else :return Response(serializedObj.errors,status=status.HTTP_400_BAD_REQUEST)
class TimeTodoApi(APIView):
    def get_object(self,pk):
        try:
            return TimeTodo.objects.get(pk=pk)
        except TimeTodo.DoesNotExist:
            return None    
    
    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.view_timetodos'), login_url='/main/login/'))        
    def get(self,request,pk): 
        time_todoObj=self.get_object(pk)
        if time_todoObj :
            #return render(request,"mainApp/time_todo.html",{"time_todo":time_todoObj,'activePage':'TimeTodos'})
            serializeobj= TimeTodoSerializer(time_todoObj)
            return Response(serializeobj.data,status=status.HTTP_200_OK)
        else : return Response(status=status.HTTP_404_NOT_FOUND)

    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.change_timetodo'), login_url='/main/login/'))
    def put(self,request,pk):
        obj=self.get_object(pk)
        serializedObj= TimeTodoSerializer(obj,data=request.data,partial=True)
        #list_error=""
        if serializedObj.is_valid():
            serializedObj.save()
            return Response(status=status.HTTP_200_OK)
        else : return Response(serializedObj.errors,status=status.HTTP_400_BAD_REQUEST)

    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.delete_timetodo'), login_url='/main/login/'))
    def delete(self,request,pk):
        obj=self.get_object(pk)
        if obj :
            try:
                obj.delete()
                return Response(status=status.HTTP_204_NO_CONTENT)
            except Exception as err_msg: 
                    return Response(data ="{}".format(str(err_msg)),status=status.HTTP_405_METHOD_NOT_ALLOWED)
        else : return Response(status=status.HTTP_404_NOT_FOUND)

###### PrescriptionApi  ########
class PrescriptionApi(APIView):
    #@method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.view_Prescription'), login_url='/main/login/'))
    def get(self,request,visit_id):
        # clinicTypeId = getClinicTypeId(request)
        # clinicTypeCode = getClinicTypeCode(request)
        # if not clinicTypeId or not clinicTypeCode: return redirect('clauthApp:home')
        context={'activePage':'PatientVisits'}
        try : patientVisitObj = PatientVisit.objects.get(pk=visit_id)
        except PatientVisit.DoesNotExist:return redirect("patientApp:PatientsApi")  
        template_name =f"Branch/brpresApp/prescription_pv.html"
        context.update({"pV":patientVisitObj})
        patient_id = patientVisitObj.patient.pk
        patient=Patient.objects.get(pk=patient_id)
        context.update({"patient":patient})
        #all data from tables 
        all_time_todos = TimeTodo.objects.filter(is_active=True)
        context.update({"time_todos":all_time_todos})
        all_drugs = Drug.objects.filter(is_active=True)
        all_drugs_srlzd=DrugMiniSerializer(all_drugs,many=True)
        context.update({"all_drugs":all_drugs_srlzd.data})
        all_medical_tests = MedicalTest.objects.filter(is_active=True)
        context.update({"all_medical_tests":all_medical_tests})
        all_radiologys = Radiology.objects.all()
        context.update({"all_radiologys":all_radiologys})
        #user prescription items 
        presDrugItems=PresDrugItem.objects.filter(patient_visit=patientVisitObj)
        presDrugItems_serialized=PresDrugItemSerializer(presDrugItems,many=True)
        context.update({"presDrugItems":presDrugItems_serialized.data})
        presRadiologyItems=PresRadiologyItem.objects.filter(patient_visit=patientVisitObj)
        presRadiologyItems_serialized=PresRadiologyItemSerializer(presRadiologyItems,many=True)
        context.update({"presRadiologyItems":presRadiologyItems_serialized.data})
        presTestItems=PresTestItem.objects.filter(patient_visit=patientVisitObj)
        presTestItems_serialized=PresTestItemSerializer(presTestItems,many=True)
        context.update({"presTestItems":presTestItems_serialized.data})
        return render(request,template_name,context)

    #@method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.Add_Prescription_Item'), login_url='/main/login/'))
    def post(self,request,visit_id):
        data = request.data
        data._mutable = True
        data.update({"patient_visit":visit_id})
        item_type=data["item_type"]
        if item_type == "1D" : serializedObj= PresDrugItemSerializer(data=data)
        elif item_type == "2T" :  serializedObj= PresTestItemSerializer(data=data)
        elif item_type == "3R" : serializedObj= PresRadiologyItemSerializer(data=data)
        else :return Response({},status=status.HTTP_400_BAD_REQUEST)
        if serializedObj.is_valid():
            serializedObj.save()
            return Response(serializedObj.data,status=status.HTTP_201_CREATED)
        else :return Response(serializedObj.errors,status=status.HTTP_400_BAD_REQUEST)
###### PrescriptionApi  ########
class PrescriptionItemApi(APIView):
    #@method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.delete_Prescription_Item'), login_url='/main/login/'))
    def delete(self,request,item_pk):
        item_type=request.data.get("item_type")
        if item_type == "Drug" : itemObj = PresDrugItem.objects.get(pk=item_pk)
        elif item_type == "Test" :  itemObj = PresTestItem.objects.get(pk=item_pk)
        elif item_type == "Radiology" : itemObj = PresRadiologyItem.objects.get(pk=item_pk)
        else : itemObj = None
        if itemObj :
            try:
                itemObj.delete()
                return Response(status=status.HTTP_204_NO_CONTENT)
            except Exception as err_msg: 
                    return Response(data ="{}".format(str(err_msg)),status=status.HTTP_405_METHOD_NOT_ALLOWED)
        else : return Response(status=status.HTTP_404_NOT_FOUND)
############## END Prescription%Module #################


class PrintPvPrescriptionApi(APIView):
    #@method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.PrintReport'), login_url='/main/login/'))
    def post(self,request):
        #request.data._mutable = True
        report_name=request.data.get("report_name")
        master_pk=request.data.get("master_pk")
        context={}
        if report_name == "prescription_pv" :#master_pk is patient visit used to collect all data
            template_name = "clinicReports/{}/prescription_pv.html".format(request.session['reports_code'])
            visit_id=master_pk
            patientVisitObj = PatientVisit.objects.get(pk=visit_id)
            context.update({"pV":patientVisitObj})
            presDrugItems=PresDrugItem.objects.filter(patient_visit=patientVisitObj)
            presDrugItems_serialized=PresDrugItemSerializer(presDrugItems,many=True)
            context.update({"presDrugItems":presDrugItems_serialized.data})
            presRadiologyItems=PresRadiologyItem.objects.filter(patient_visit=patientVisitObj)
            presRadiologyItems_serialized=PresRadiologyItemSerializer(presRadiologyItems,many=True)
            context.update({"presRadiologyItems":presRadiologyItems_serialized.data})
            presTestItems=PresTestItem.objects.filter(patient_visit=patientVisitObj)
            presTestItems_serialized=PresTestItemSerializer(presTestItems,many=True)
            context.update({"presTestItems":presTestItems_serialized.data})
            return render(request,template_name,context)